package controller 
{
	import gameobjects.Dragon;
	import gameobjects.Level;

	import flash.display.DisplayObjectContainer;
	import flash.filters.GlowFilter;
	
	/**
	 * Keeps the gamestate, and the controllers
	 * 
	 * @author GS
	 */
	public final class GameController 
	{
		private var _gameStage:DisplayObjectContainer;
		private var _dragonController:DragonController;
		private var _serverConnection:ConnectionController;
		
		private var dragonsKeeper:Object;
		private var ownId:int;
		
		public function GameController(gameStage:DisplayObjectContainer) 
		{
			this._gameStage = gameStage;
			this._dragonController = new DragonController();
			this._serverConnection = new ConnectionController();
			this.dragonsKeeper = new Object();
			
			this._serverConnection.addEventListener(ConnectionEvent.CONNECT, onConnect);
			this._serverConnection.addEventListener(ConnectionEvent.UPDATE, onUpdate);
			this._gameStage.addChild(new Level("xml/level.xml"));
			
		}
		
		private function onConnect(e:ConnectionEvent):void 
		{
			addPlayer(e.dragonDelta.id, e.dragonDelta.x, e.dragonDelta.y, true);
			ownId = e.dragonDelta.id;
		}
		
		private function onUpdate(e:ConnectionEvent):void 
		{
			if (e.dragonDelta.id == ownId) return;
			
			var dragon:Dragon= dragonsKeeper[e.dragonDelta.id];
			if (!dragon) 
			{
				dragon = addPlayer(e.dragonDelta.id, e.dragonDelta.x, e.dragonDelta.y);
			}
			
			dragon.x = e.dragonDelta.x;
			dragon.y = e.dragonDelta.y;
			dragon.action = e.dragonDelta.action;
		}
		
		public function addPlayer(id:int, x:int, y:int, isPlayer:Boolean = false):Dragon
		{
			var d:Dragon = new Dragon(id);
			d.x = x;
			d.y = y;
			d.addEventListener(DragonEvent.KILL, onKill);
			
			if (isPlayer)
			{
				this._dragonController.initDragon(d, this._gameStage.stage);
				this._serverConnection.initPlayer(d);
				d.filters = [new GlowFilter(0xAA9999)];
			}
			else
			{
				this._dragonController.initDragon(d);
				dragonsKeeper[id] = d;
			}
			
			this._gameStage.addChild(d);
			
			return d;
		}
		
		private function onKill(e:DragonEvent):void 
		{
			var d:Dragon = Dragon(e.target);
			trace(d, d.id, "scored!");
		}
		
		
	}

}