package gameobjects 
{
	import controller.DragonAction;
	import controller.DragonEvent;

	import flash.display.DisplayObject;
	import flash.utils.setTimeout;
	
	/**
	 * The players Dragon Object, Animation wrapper, actionkeeper, ... everything
	 * 
	 * @author GS
	 */
	public final class Dragon extends GameObject 
	{
		private var _action:int;
		private var _active:Boolean;
		private var _falling:Boolean;
		private var _deltaX:int;
		private var _deltaY:int;
		private var _bubble:Bubble;
		
		public var id:int;
		public var score:int;
		
		public function Dragon(id:int) 
		{
			super("gfx/dragon/dragon.swf");
			
			this.id = id;
		}
		
		override protected function init():void 
		{
			this.cc.addDragon(this);
			
			this._bubble = new Bubble();
			
			this.action = DragonAction.FALL;
			_active = true;
		}
		
		public function left():void
		{
			if (_action == DragonAction.LEFT || !_active) return;
			
			dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.LEFT));
			_deltaX = -5;
			this.scaleX = -1;
			this.x += this.width / 3;
			gotoAndPlay("move");
			_action = DragonAction.LEFT;
		}
		
		public function right():void
		{
			if (_action == DragonAction.RIGHT || !_active) return;
			
			dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.RIGHT));
			_deltaX = 5;
			this.scaleX = 1;
			this.x -= this.width / 3;
			gotoAndPlay("move");
			_action = DragonAction.RIGHT;
		}
		
		public function stop():void
		{
			dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.STOP));
			_deltaX = 0;
			gotoAndStop("move");
			_action = DragonAction.STOP;
		}
		
		public function jump():void
		{
			if (!_falling && _active) 
			{
				dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.JUMP));
				_deltaY = -15;
				fall();
			}
		}
		
		public function fall():void
		{
			gotoAndPlay("fall");
			_falling = true;
		}
		
		public function drop():void
		{
			_deltaY = 0;
			_falling = false;
		}
		
		public function bobble():void
		{
			gotoAndStop("bobble");
			
			if (_action != DragonAction.BOBBEL) 
			{
				dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.BOBBEL));
				setTimeout(releaseBobble, 2000);
				_deltaX = 0;
				_deltaY = -4;
				_falling = false;
				_active = false;
				_action = DragonAction.BOBBEL;
			}
		}
		
		public function releaseBobble():void
		{
			_active = true;
			fall();
		}
		
		public function die():void
		{
			dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.DEAD));
			setTimeout(respawn, 2000);
			_deltaY = 10;
			gotoAndPlay("die");
			_action = DragonAction.DEAD;
			_active = false;
		}
		
		public function respawn():void
		{
			x = 400;
			y = 0;
			spawn();
		}
		
		public function spawn():void
		{
			dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.SPAWN));
			_active = true;
			fall();
		}
		
		public function shot():void
		{
			if (!_active) return;
			
			dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.SHOT));
			_bubble.shot(this);
			gotoAndPlay("shot");
		}
		
		public function kill():void
		{
			dispatchEvent(new DragonEvent(DragonEvent.ACTION, DragonAction.KILL));
			dispatchEvent(new DragonEvent(DragonEvent.KILL));
			score++;
			//dispatchEvent(new Event("kill")); //auch möglich aber schlecht
		}
		
		public function tick():void
		{
			x += _deltaX;
			y += _deltaY;
			
			switch(_action)
			{
				case DragonAction.LEFT :
				case DragonAction.RIGHT :
					var obs:Obstacle = cc.collisionWithObstacles(this);
					if (obs == null) 
					{
						fall();
					}
					else if (obs.border) 
					{
						x -= _deltaX;
						stop();
					}
				break;
				case DragonAction.BOBBEL :
					var killer:Dragon = cc.collisionWithDragons(this);
					
					if (killer != null)
					{
						trace(this.id, "Bobble kill", this._action);
						killer.action = DragonAction.KILL;
						die();
						return;
					}
				break;
			}
			
			if(_falling)
			{
				_deltaY++; //drop
				
				if (_deltaY > 0) var collision:DisplayObject = cc.collisionWithObstacles(this); 
				
				if (collision != null) 
				{
					y = collision.y - height; //Drop on Obstacle
					drop();
				}
			}
		}
		
		//State setter/getter
		public function get action():int
		{
			return _action;
		}
		
		public function set action(value:int):void 
		{
			switch(value)
			{
				case DragonAction.STOP : stop();
				break;
				case DragonAction.LEFT : left();
				break;
				case DragonAction.RIGHT : right();
				break;
				case DragonAction.JUMP : jump();
				break;
				case DragonAction.FALL : fall();
				break;
				case DragonAction.SHOT : shot();
				break;
				case DragonAction.BOBBEL : bobble();
				break;
				case DragonAction.DEAD : die();
				break;
				case DragonAction.KILL : kill();
				break;
			}
		}
	}

}